@@ -2,6 +2,7 @@
 
 from . import filters
 from . import global_context
+from . import extensions
 
-__all__ = ['filters', 'global_context']
+__all__ = ['filters', 'global_context', 'extensions']
 
